//
//  ListaDupla.h
//  Class Template "ListaDupla"
//
//  Created by Carlos Santos
//

#ifndef ListaDupla_h
#define ListaDupla_h

#pragma onde
#include <iostream>
#include <stdlib.h>
#include "NoDuplo.h"
#include "Iterador.h"

template <class T>
class ListaDupla {
private:
    // Declara a cabeça e a cauda como nós
    NoDuplo<T> *cabeca;
    NoDuplo<T> *cauda;
    
public:
    // O construtor inicia a cabeça e a cauda como novos nós vazios
    ListaDupla() {
        cabeca = new NoDuplo<T>();
        cauda = new NoDuplo<T>();
    }
    // O destrutor deverá limpar a lista e libertar os nós da cabeça e da cauda
    ~ListaDupla() {
        limpar();
        delete cabeca;
        delete cauda;
    }
    
    // Devolver "true" se a lista estiver vazia
    bool vazia() { return (cabeca->seguinte == NULL); }
    
    // Insere um nó à cabeça da lista
    void inserePrimeiro(T &e) {

        if (vazia()) {
            // Se a cabeça estiver vazia, insere um "primeiro" nó
            NoDuplo<T> *novoNo = new NoDuplo<T>(e);
            cabeca->seguinte = novoNo;
            cauda->anterior = novoNo;
        }
        else {
            // Caso contrário, insere um novo "primeiro" nó
            NoDuplo<T> *primeiroActual = cabeca->seguinte;
            // O novo nó deverá ter o apontador "seguinte" a apontar para antigo primeiro nó...
            // ... e o apontador "anterior" a NULL pois será o novo primeiro nó
            NoDuplo<T> *novoNo = new NoDuplo<T>(e, primeiroActual);
            // O antigo primeiro nó deverá ficar com o apontador "anterior" a apontar para o novo primeiro nó
            primeiroActual->anterior = novoNo;
            // Finalmente, a cabeça deverá apontar para o novo nó
            cabeca->seguinte = novoNo;
        }
        
    }
    
    // Insere um nó à cauda da lista
    void insereUltimo(T &e) {
        
        if (vazia()) {
            // Se a cabeça estiver vazia, insere um "primeiro" nó
            NoDuplo<T> *novoNo = new NoDuplo<T>(e);
            cabeca->seguinte = novoNo;
            cauda->anterior = novoNo;
        }
        else {
            // Caso contrário, insere um novo "último" nó
            NoDuplo<T> *ultimoActual = cauda->anterior;
            // O novo nó deverá ter o apontador "seguinte" a NULL pois será o novo último nó ...
            // ... e o apontador "anterior" a apontar para o antigo último nó
            NoDuplo<T> *novoNo = new NoDuplo<T>(e, NULL, ultimoActual);
            // O antigo último nó deverá ficar com o apontador "seguinte" a apontar para o novo último nó
            ultimoActual->seguinte = novoNo;
            // Finalmente, a cauda deverá apontar para o novo últmo nó
            cauda->anterior = novoNo;
        }
        
    }
    
    // Insere um nó antes de um determinado nó de "referência"
    bool insereAntesDe(NoDuplo<T> *ref, T &e) {
        
        if (vazia()) {
            // Se a lista estiver vazia, devolve "false"
            return false;
        }

        // Procura o nó de "referência" a partir da cabeça
        NoDuplo<T> *procuraRef = cabeca->seguinte;
        while (procuraRef!=NULL) {
            if (procuraRef==ref) {
                // Se encontrar o nó, interrompe o ciclo
                break;
            }
            procuraRef = procuraRef->seguinte;
        }

        if (procuraRef==NULL) {
            // Se o nó de "referência" não foi encontrado, retorna "false"
            return false;
        }
        else {
            // Se o encontrou, declara um apontador para a posição antecedente ao nó de referência
            NoDuplo<T> *antesRef = procuraRef->anterior;
            // O novo nó deverá ter o apontador "seguinte" a apontar para o nó de referência (insere antes de) ...
            // ... e o apontador "anterior" para o nó antecedente
            NoDuplo<T> *novoNo = new NoDuplo<T>(e, procuraRef, antesRef);

            if (antesRef==NULL) {
                // Se o apontador do nó antecedente for NULL, estamos na cabeça da lista - Insere à cabeça
                cabeca->seguinte = novoNo;
            }
            else {
                // Caso contrário, altera o apontador "seguinte" do nó antecedente para o novo nó
                antesRef->seguinte = novoNo;
            }
            // Altera o apontador "anterior" do nó de referência para o novo nó
            procuraRef->anterior = novoNo;
            return true;
        }
        
    }
    
    // Insere um nó depois de um determinado nó de "referência"
    bool insereDepoisDe(NoDuplo<T> *ref, T &e) {
        
        if (vazia()) {
            // Se a lista estiver vazia, devolve "false"
            return false;
        }
        
        // Procura o nó de "referência" a partir da cabeça
        NoDuplo<T> *procuraRef = cabeca->seguinte;
        while (procuraRef!=NULL) {
            if (procuraRef==ref) {
                // Se encontrar o nó, interrompe o ciclo
                break;
            }
            procuraRef = procuraRef->seguinte;
        }

        if (procuraRef==NULL) {
            // Se o nó de "referência" não foi encontrado, retorna "false"
            return false;
        }
        else {
            // Se o encontrou, declara um apontador para a posição seguinte ao nó de referência
            NoDuplo<T> *depoisRef = procuraRef->seguinte;
            // O novo nó deverá ter o apontador "seguinte" a apontar para o nó depois da referência ...
            // ... e o apontador "anterior" para o nó de referência (insere depois de)
            NoDuplo<T> *novoNo = new NoDuplo<T>(e, depoisRef, procuraRef);
            
            if (depoisRef==NULL) {
                // Se o apontador do nó seguinte for NULL, estamos na cauda da lista - Insere à cauda
                cauda->anterior = novoNo;
            }
            else {
                // Caso contrário, altera o apontador "anterior" do nó seguinte para o novo nó
                depoisRef->anterior = novoNo;
            }
            // Altera o apontador "seguinte" do nó de referência para o novo nó
            procuraRef->seguinte = novoNo;
            return true;
        }
        
    }
    
    // Remove da lista um dado nó
    bool remove(NoDuplo<T> *r) {
        
        if (vazia()) {
            // Se a lista estiver vazia, devolve "false"
            return false;;
        }
        
        // Define um apontador para procurar o nó a remover
        NoDuplo<T> *remover = cauda->anterior;
        while (remover!=NULL) {
            if (remover==r) {
                // Se encontrar o nó, interrompe o ciclo
                break;
            }
            remover = remover->anterior;
        }
        
        if (remover==NULL) {
            // Se o nó não foi encontrado, retorna "false"
            return false;
        }
        else {
            // Caso contrário, declara dois apontadores para os nós que estão depois e antes do nó a remover
            NoDuplo<T> *depoisRemover = remover->seguinte;
            NoDuplo<T> *antesRemover = remover->anterior;
            if (depoisRemover==NULL) {
                // Se o apontador do nó a seguir ao nó a remover for NULL estamos a removar à cauda
                cauda->anterior = antesRemover;
            }
            else {
                // Caso contrário, altera o apontador "anterior" do nó a seguir ao nó a remover para o seu nó antecedente
                depoisRemover->anterior = antesRemover;
            }
            if (antesRemover==NULL) {
                // Se o apontador do nó anterior ao nó a remover for NULL estamos a remover à cabeça
                cabeca->seguinte = depoisRemover;
            }
            else {
                // Caso contrário, altera o apontador "seguinte" do nó anterior ao nó a remover para o seu nó posterior
                antesRemover->seguinte = depoisRemover;
            }
            // Liberta o nó a remover da memória
            delete remover;
            return true;
        }
    }
    
    // Limpa todos os nós da lista
    void limpar() {
        
        int contador = 0;
        
        // Remove sucessivamente os nós à cauda até ao último nó
        while (cauda->anterior!=NULL) {
            // Obtem o nó a remover a partir da cauda
            NoDuplo<T> *remove = cauda->anterior;
            // Copia o apontador "anterior" do nó a remover para a cauda
            cauda->anterior = remove->anterior;
            // Liberta o nó a remover da memória
            delete remove;
            contador++;
        }
        // Por regra, uma classe não escreve nada no ecrã. Se usar esta classe num projecto seu
        // deverá de apagar a seguinte linha e o contador depois dos testes.
        std::cout << "Libertados da memória " << contador << " nós" << std::endl;
    }
    
    // Devolve um novo iterador iniciado à cabeça
    Iterador<T> *obtemIteradorCabeca() {
        Iterador<T> *i = new Iterador<T>(cabeca);
        return i;
    }
    
    // Devolve um novo iterador iniciado à cauda
    Iterador<T> *obtemIteradorCauda() {
        Iterador<T> *i = new Iterador<T>(cauda);
        return i;
    }
};

#endif
