//
//  agenda.cpp
//  Define uma lista com uma agenda com nomes, telefones e e-mails por ordem crescente de nome
//  Este exemplo usa os seguintes métodos da class ListaDupla:
//  - vazia
//  - inserePrimeiro
//  - insereUltimo
//  - insereAntesDe
//  - remove
//  - obtemIteradorCabeca
//  - obtemIteradorCauda
//
//  Created by Carlos Santos
//

#include "agenda.h"

void agenda() {
    
    ListaDupla<Agenda> *listaAgenda = new ListaDupla<Agenda>();
    char menu = ' ';
    string nome;
    string telefone;
    string email;
    
    while (menu!='0') {
        
        cout << endl << "=== Lista Agenda ===" << endl;
        cout << "1 - Inserir contacto" << endl;
        cout << "2 - Remover contacto" << endl;
        cout << "3 - Listar contactos" << endl;
        cout << "4 - Listar contactos por ordem inversa" << endl;
        cout << "0 - Sair da agenda" << endl;
        if (!(cin >> menu)) {
            // Se foram introduzidos caracteres incorrectos, limpa o estado de erro e ignora os caracteres
            cin.clear();
            cin.ignore(10000, '\n');
            menu = ' ';
        }
        
        switch (menu) {
            case '0':
                break;
            case '1': {
                cout << "Introduza o nome: ";
                cin >> nome;
                cout << "Introduza o telefone: ";
                cin >> telefone;
                cout << "Introduza o e-mail: ";
                cin >> email;
                
                // Declara uma variável do tipo Agenda e carrega os dados inseridos
                Agenda agenda;
                agenda.nome = nome;
                agenda.telefone = telefone;
                agenda.email = email;
                
                if (listaAgenda->vazia()) {
                    // Se a lista estiver vazia, insere um primeiro nó à cabeça
                    listaAgenda->inserePrimeiro(agenda);
                }
                else {
                    // Caso contrário, obtem o iterador da lista a partir da cabeça
                    Iterador<Agenda> *i = listaAgenda->obtemIteradorCabeca();
                    NoDuplo<Agenda> *procura;
                    // Procura o nó com um contacto em que o nome seja "maior" do que o que queremos inserir
                    while ((procura=i->noSeguinte())!=NULL) {
                        Agenda a = procura->elemento;
                        if (a.nome>nome) {
                            // Se encontrou um nome "maior", interrompe o ciclo
                            break;
                        }
                    }
                    
                    if (procura==NULL) {
                        // Se não encontrou um nome maior, insere o novo contacto à cauda
                        listaAgenda->insereUltimo(agenda);
                    }
                    else {
                        // Caso contrário, insere o novo contacto antes do nó com o nome "maior"
                        listaAgenda->insereAntesDe(procura, agenda);
                    }
                    delete i;
                }
                break;
            }
            case '2': {
                cout << "Introduza o nome a remover: ";
                cin >> nome;
                
                Agenda agenda;
                // Obtem um iterador a partir da cabeça
                Iterador<Agenda> *i = listaAgenda->obtemIteradorCabeca();
                NoDuplo<Agenda> *procura;
                // Procura o nó com o contacto com o nome que queremos remover
                while ((procura=i->noSeguinte())!=NULL) {
                    agenda = procura->elemento;
                    if (agenda.nome==nome) {
                        // Se encontrou o nome, interrompe o ciclo
                        break;
                    }
                }
                
                if (procura==NULL) {
                    // Se o nome não foi encontrado...
                    cout << "O nome não foi encontrado na agenda" << endl;
                }
                else {
                    // Se encontrou o nome, tenta removê-lo da lista
                    if (listaAgenda->remove(procura)) {
                        cout << "Contacto removido com sucesso" << endl;
                    }
                    else {
                        cout << "Erro ao remover contacto" << endl;
                    }
                }
                // Liberta o iterador
                delete i;
                break;
            }
            case '3':
                if (listaAgenda->vazia()) {
                    cout << "A lista está vazia" << endl;
                }
                else {
                    cout << endl << "Lista de contactos" << endl;
                    // Obtem o iterador a partir da cabeça
                    Iterador<Agenda> *i = listaAgenda->obtemIteradorCabeca();
                    NoDuplo<Agenda> *lista;
                    // Lista todos os nós até ao fim da lista
                    while ((lista=i->noSeguinte())!=NULL) {
                        cout << "Nome: " << lista->elemento.nome << endl;
                        cout << "Telefone: " << lista->elemento.telefone << endl;
                        cout << "E-Mail: " << lista->elemento.email << endl << endl;
                    }
                    // Liberta o iterador
                    delete i;
                }
                break;
            case '4':
                if (listaAgenda->vazia()) {
                    cout << "A lista está vazia" << endl;
                }
                else {
                    cout << endl << "Lista de contactos por ordem inversa" << endl;
                    // Obtem o iterador a partir da cauda
                    Iterador<Agenda> *i = listaAgenda->obtemIteradorCauda();
                    NoDuplo<Agenda> *lista;
                    // Lista todos os nós do fim para o início da lista
                    while ((lista=i->noAnterior())!=NULL) {
                        cout << "Nome: " << lista->elemento.nome << endl;
                        cout << "Telefone: " << lista->elemento.telefone << endl;
                        cout << "E-Mail: " << lista->elemento.email << endl << endl;
                    }
                    // Liberta o iterador
                    delete i;
                }
            default:
                cout << "Opção inválida" << endl;
                break;
        }
    }
    
    // Liberta a lista da memória
    delete listaAgenda;
    
}